@@ -264,6 +264,9 @@ inline void debug_ast(AST_Node* node, string ind = "", Env* env = 0)
     cerr << ind << "If " << block;
     cerr << " (" << pstate_source_position(node) << ")";
     cerr << " " << block->tabs() << endl;
+    debug_ast(block->predicate(), ind + " = ");
+    debug_ast(block->consequent(), ind + " <>");
+    debug_ast(block->alternative(), ind + " ><");
   } else if (dynamic_cast<Return*>(node)) {
     Return* block = dynamic_cast<Return*>(node);
     cerr << ind << "Return " << block;
@@ -290,6 +293,7 @@ inline void debug_ast(AST_Node* node, string ind = "", Env* env = 0)
     cerr << ind << "Import " << block;
     cerr << " (" << pstate_source_position(node) << ")";
     cerr << " " << block->tabs() << endl;
+    // debug_ast(block->media_queries(), ind + " @ ");
     // vector<string>         files_;
     for (auto imp : block->urls()) debug_ast(imp, "@ ", env);
   } else if (dynamic_cast<Assignment*>(node)) {
@@ -352,7 +356,9 @@ inline void debug_ast(AST_Node* node, string ind = "", Env* env = 0)
     if (block->block()) for(auto i : block->block()->elements()) { debug_ast(i, ind + " ", env); }
   } else if (dynamic_cast<Mixin_Call*>(node)) {
     Mixin_Call* block = dynamic_cast<Mixin_Call*>(node);
-    cerr << ind << "Mixin_Call " << block << " " << block->tabs() << endl;
+    cerr << ind << "Mixin_Call " << block << " " << block->tabs();
+    cerr << " [" <<  block->name() << "]" << endl;
+    debug_ast(block->arguments(), ind + " args: ");
     if (block->block()) for(auto i : block->block()->elements()) { debug_ast(i, ind + " ", env); }
   } else if (dynamic_cast<Ruleset*>(node)) {
     Ruleset* ruleset = dynamic_cast<Ruleset*>(node);
@@ -374,7 +380,9 @@ inline void debug_ast(AST_Node* node, string ind = "", Env* env = 0)
     else if (expression->type() == Textual::PERCENTAGE) cerr << " [PERCENTAGE]";
     else if (expression->type() == Textual::DIMENSION) cerr << " [DIMENSION]";
     else if (expression->type() == Textual::HEX) cerr << " [HEX]";
-    cerr << expression << " [" << expression->value() << "]" << endl;
+    cerr << expression << " [" << expression->value() << "]";
+    if (expression->is_delayed()) cerr << " [delayed]";
+    cerr << endl;
   } else if (dynamic_cast<Variable*>(node)) {
     Variable* expression = dynamic_cast<Variable*>(node);
     cerr << ind << "Variable " << expression;
@@ -398,6 +406,7 @@ inline void debug_ast(AST_Node* node, string ind = "", Env* env = 0)
   } else if (dynamic_cast<Arguments*>(node)) {
     Arguments* expression = dynamic_cast<Arguments*>(node);
     cerr << ind << "Arguments " << expression;
+    if (expression->is_delayed()) cerr << " [delayed]";
     cerr << " (" << pstate_source_position(node) << ")";
     cerr << endl;
     for(auto i : expression->elements()) { debug_ast(i, ind + " ", env); }
@@ -435,7 +444,7 @@ inline void debug_ast(AST_Node* node, string ind = "", Env* env = 0)
     Binary_Expression* expression = dynamic_cast<Binary_Expression*>(node);
     cerr << ind << "Binary_Expression " << expression;
     cerr << " (" << pstate_source_position(node) << ")";
-    cerr << " [" << expression->type() << "]" << endl;
+    cerr << " [" << expression->type_name() << "]" << endl;
     debug_ast(expression->left(), ind + " left:  ", env);
     debug_ast(expression->right(), ind + " right: ", env);
   } else if (dynamic_cast<Map*>(node)) {
@@ -443,6 +452,10 @@ inline void debug_ast(AST_Node* node, string ind = "", Env* env = 0)
     cerr << ind << "Map " << expression;
     cerr << " (" << pstate_source_position(node) << ")";
     cerr << " [Hashed]" << endl;
+    // for (auto i : expression->elements()) {
+    //   debug_ast(i.first, ind + " key: ");
+    //   debug_ast(i.second, ind + " val: ");
+    // }
   } else if (dynamic_cast<List*>(node)) {
     List* expression = dynamic_cast<List*>(node);
     cerr << ind << "List " << expression;
@@ -476,37 +489,37 @@ inline void debug_ast(AST_Node* node, string ind = "", Env* env = 0)
     cerr << " [" << expression->value() << expression->unit() << "]" << endl;
   } else if (dynamic_cast<String_Quoted*>(node)) {
     String_Quoted* expression = dynamic_cast<String_Quoted*>(node);
-    cerr << ind << "String_Quoted : " << expression;
+    cerr << ind << "String_Quoted " << expression;
     cerr << " (" << pstate_source_position(node) << ")";
-    cerr << " [" << prettyprint(expression->value()) << "]" <<
-      (expression->is_delayed() ? " {delayed}" : "") <<
-      (expression->sass_fix_1291() ? " {sass_fix_1291}" : "") <<
-      (expression->quote_mark() != 0 ? " {qm:" + string(1, expression->quote_mark()) + "}" : "") <<
-      " <" << prettyprint(expression->pstate().token.ws_before()) << ">" << endl;
+    cerr << " [" << prettyprint(expression->value()) << "]";
+    if (expression->is_delayed()) cerr << " [delayed]";
+    if (expression->sass_fix_1291()) cerr << " [sass_fix_1291]";
+    if (expression->quote_mark()) cerr << " [quote_mark: " << expression->quote_mark() << "]";
+    cerr << " <" << prettyprint(expression->pstate().token.ws_before()) << ">" << endl;
   } else if (dynamic_cast<String_Constant*>(node)) {
     String_Constant* expression = dynamic_cast<String_Constant*>(node);
-    cerr << ind << "String_Constant : " << expression;
+    cerr << ind << "String_Constant " << expression;
+    cerr << " " << expression->concrete_type() <<
     cerr << " (" << pstate_source_position(node) << ")";
-    cerr << " [" << prettyprint(expression->value()) << "]" <<
-      (expression->is_delayed() ? " {delayed}" : "") <<
-      (expression->sass_fix_1291() ? " {sass_fix_1291}" : "") <<
-      (expression->quote_mark() != 0 ? " {qm:" + string(1, expression->quote_mark()) + "}" : "") <<
-      " <" << prettyprint(expression->pstate().token.ws_before()) << ">" << endl;
+    cerr << " [" << prettyprint(expression->value()) << "]";
+    if (expression->is_delayed()) cerr << " [delayed]";
+    if (expression->sass_fix_1291()) cerr << " [sass_fix_1291]";
+    cerr << " <" << prettyprint(expression->pstate().token.ws_before()) << ">" << endl;
   } else if (dynamic_cast<String_Schema*>(node)) {
     String_Schema* expression = dynamic_cast<String_Schema*>(node);
     cerr << ind << "String_Schema " << expression;
-    cerr << " (" << pstate_source_position(node) << ")";
-    cerr << " " << expression->concrete_type() <<
-      (expression->has_interpolants() ? " {has_interpolants}" : "") <<
-      endl;
+    cerr << " " << expression->concrete_type();
+    if (expression->is_delayed()) cerr << " [delayed]";
+    if (expression->has_interpolants()) cerr << " [has_interpolants]";
+    cerr << " <" << prettyprint(expression->pstate().token.ws_before()) << ">" << endl;
     for(auto i : expression->elements()) { debug_ast(i, ind + " ", env); }
   } else if (dynamic_cast<String*>(node)) {
     String* expression = dynamic_cast<String*>(node);
     cerr << ind << "String " << expression;
+    cerr << " " << expression->concrete_type();
     cerr << " (" << pstate_source_position(node) << ")";
-    cerr << expression->concrete_type() <<
-      " " << (expression->sass_fix_1291() ? "{sass_fix_1291}" : "") <<
-      endl;
+    if (expression->sass_fix_1291()) cerr << " [sass_fix_1291]";
+    cerr << " <" << prettyprint(expression->pstate().token.ws_before()) << ">" << endl;
   } else if (dynamic_cast<Expression*>(node)) {
     Expression* expression = dynamic_cast<Expression*>(node);
     cerr << ind << "Expression " << expression;
