@@ -8,6 +8,18 @@ namespace Sass {
   : type(type), pstate(pstate), message(message)
   { }
 
+  void warn(string msg, ParserState pstate)
+  {
+    cerr << "Warning: " << msg<< endl;
+  }
+
+  void warn(string msg, ParserState pstate, Backtrace* bt)
+  {
+    Backtrace top(bt, pstate, "");
+    msg += top.to_string();
+    warn(msg, pstate);
+  }
+
   void error(string msg, ParserState pstate)
   {
     throw Sass_Error(Sass_Error::syntax, pstate, msg);
@@ -15,11 +27,9 @@ namespace Sass {
 
   void error(string msg, ParserState pstate, Backtrace* bt)
   {
-
     Backtrace top(bt, pstate, "");
-    msg += top.to_string();
-
-    throw Sass_Error(Sass_Error::syntax, pstate, msg);
+    msg += "\n" + top.to_string();
+    error(msg, pstate);
   }
 
 }
