@@ -1,3 +1,10 @@
+#include <cstdlib>
+#include <cmath>
+#include <iostream>
+#include <sstream>
+#include <iomanip>
+#include <typeinfo>
+
 #include "file.hpp"
 #include "eval.hpp"
 #include "ast.hpp"
@@ -11,12 +18,6 @@
 #include "prelexer.hpp"
 #include "parser.hpp"
 
-#include <cstdlib>
-#include <cmath>
-#include <iostream>
-#include <iomanip>
-#include <typeinfo>
-
 namespace Sass {
   using namespace std;
 
@@ -397,6 +398,13 @@ namespace Sass {
     for (auto key : m->keys()) {
       *mm << std::make_pair(key->perform(this), m->at(key)->perform(this));
     }
+
+    // check for duplicate keys
+    if (mm->has_duplicate_key()) {
+      To_String to_string(&ctx);
+      error("Duplicate key \"" + mm->get_duplicate_key()->perform(&to_string) + "\" in map " + m->perform(&to_string) + ".", m->pstate());
+    }
+
     mm->is_expanded(true);
     return mm;
   }
@@ -435,8 +443,23 @@ namespace Sass {
     }
     // not a logical connective, so go ahead and eval the rhs
     Expression* rhs = b->right()->perform(this);
-    rhs->is_delayed(false);
-    while (typeid(*rhs) == typeid(Binary_Expression)) rhs = rhs->perform(this);
+    // maybe fully evaluate structure
+    if (op_type == Binary_Expression::EQ ||
+        op_type == Binary_Expression::NEQ ||
+        op_type == Binary_Expression::GT ||
+        op_type == Binary_Expression::GTE ||
+        op_type == Binary_Expression::LT ||
+        op_type == Binary_Expression::LTE)
+    {
+      rhs->is_expanded(false);
+      rhs->set_delayed(false);
+      rhs = rhs->perform(this);
+    }
+    else
+    {
+      rhs->is_delayed(false);
+      rhs = rhs->perform(this);
+    }
 
     // see if it's a relational expression
     switch(op_type) {
@@ -512,6 +535,11 @@ namespace Sass {
 
   Expression* Eval::operator()(Function_Call* c)
   {
+    if (backtrace->parent != NULL && backtrace->depth() > Constants::MaxCallStack) {
+        ostringstream stm;
+        stm << "Stack depth exceeded max of " << Constants::MaxCallStack;
+        error(stm.str(), c->pstate(), backtrace);
+    }
     string name(Util::normalize_underscores(c->name()));
     string full_name(name + "[f]");
     Arguments* args = c->arguments();
@@ -789,6 +817,7 @@ namespace Sass {
 
   Expression* Eval::operator()(Number* n)
   {
+    n->normalize();
     // behave according to as ruby sass (add leading zero)
     return new (ctx.mem) Number(n->pstate(),
                                 n->value(),
@@ -857,6 +886,7 @@ namespace Sass {
       Expression* value = static_cast<Expression*>((*env)[name]);
       return evacuate_quotes(interpolation(value));
     } else if (Binary_Expression* var = dynamic_cast<Binary_Expression*>(s)) {
+      var->is_delayed(false);
       Expression* ex = var->perform(this);
       return evacuate_quotes(interpolation(ex));
     } else if (Function_Call* var = dynamic_cast<Function_Call*>(s)) {
@@ -882,9 +912,9 @@ namespace Sass {
   {
     string acc;
     for (size_t i = 0, L = s->length(); i < L; ++i) {
-      if (String_Quoted* str_quoted = dynamic_cast<String_Quoted*>((*s)[i])) {
-        if (!str_quoted->is_delayed()) str_quoted->value(string_eval_escapes(str_quoted->value()));
-      }
+      // if (String_Quoted* str_quoted = dynamic_cast<String_Quoted*>((*s)[i])) {
+        // if (!str_quoted->is_delayed()) str_quoted->value(string_eval_escapes(str_quoted->value()));
+      // }
       acc += interpolation((*s)[i]);
     }
     String_Quoted* str = new (ctx.mem) String_Quoted(s->pstate(), acc);
@@ -995,8 +1025,7 @@ namespace Sass {
         is_rest_argument = false;
         is_keyword_argument = true;
       }
-      else
-      if(val->concrete_type() != Expression::LIST) {
+      else if(val->concrete_type() != Expression::LIST) {
         List* wrapper = new (ctx.mem) List(val->pstate(),
                                            0,
                                            List::COMMA,
@@ -1028,9 +1057,13 @@ namespace Sass {
 
   Expression* Eval::operator()(Parent_Selector* p)
   {
+    // no idea why both calls are needed
     Selector* s = p->perform(contextualize);
+    if (!s) s = p->selector()->perform(contextualize);
     // access to parent selector may return 0
     Selector_List* l = static_cast<Selector_List*>(s);
+    // some spec tests cause this (might be a valid case!)
+    // if (!s) { cerr << "Parent Selector eval error" << endl; }
     if (!s) { l = new (ctx.mem) Selector_List(p->pstate()); }
     return l->perform(listize);
   }
@@ -1055,13 +1088,8 @@ namespace Sass {
       } break;
 
       case Expression::NUMBER: {
-        Number* l = static_cast<Number*>(lhs);
-        Number* r = static_cast<Number*>(rhs);
-        Number tmp_r(*r);
-        tmp_r.normalize(l->find_convertible_unit());
-        return l->unit() == tmp_r.unit() && l->value() == tmp_r.value()
-               ? true
-               : false;
+        return *static_cast<Number*>(lhs) ==
+               *static_cast<Number*>(rhs);
       } break;
 
       case Expression::COLOR: {
@@ -1152,8 +1180,8 @@ namespace Sass {
       v->denominator_units() = r->denominator_units();
     }
 
-    v->value(ops[op](lv, tmp.value()));
     if (op == Binary_Expression::MUL) {
+      v->value(ops[op](lv, rv));
       for (size_t i = 0, S = r->numerator_units().size(); i < S; ++i) {
         v->numerator_units().push_back(r->numerator_units()[i]);
       }
@@ -1162,12 +1190,15 @@ namespace Sass {
       }
     }
     else if (op == Binary_Expression::DIV) {
+      v->value(ops[op](lv, rv));
       for (size_t i = 0, S = r->numerator_units().size(); i < S; ++i) {
         v->denominator_units().push_back(r->numerator_units()[i]);
       }
       for (size_t i = 0, S = r->denominator_units().size(); i < S; ++i) {
         v->numerator_units().push_back(r->denominator_units()[i]);
       }
+    } else {
+      v->value(ops[op](lv, tmp.value()));
     }
     v->normalize();
     return v;
