@@ -916,28 +916,25 @@ namespace Sass {
       string newstr;
       try {
         String_Constant* s = ARG("$string", String_Constant);
-        Number* n = ARG("$start-at", Number);
-        Number* m = ARG("$end-at", Number);
+        double start_at = ARG("$start-at", Number)->value();
+        double end_at = ARG("$end-at", Number)->value();
 
         string str = unquote(s->value());
 
-        // normalize into 0-based indices
-        size_t start = UTF_8::offset_at_position(str, UTF_8::normalize_index(static_cast<int>(n->value()), UTF_8::code_point_count(str)));
-        size_t end = UTF_8::offset_at_position(str, UTF_8::normalize_index(static_cast<int>(m->value()), UTF_8::code_point_count(str)));
+        size_t size = utf8::distance(str.begin(), str.end());
+        if (end_at <= size * -1.0) { end_at += size; }
+        if (end_at < 0) { end_at += size + 1; }
+        if (end_at > size) { end_at = size; }
+        if (start_at < 0) { start_at += size + 1; }
+        else if (start_at == 0) { ++ start_at; }
 
-        // `str-slice` should always return an empty string when $end-at == 0
-        // `normalize_index` normalizes 1 -> 0 so we need to check the original value
-        if(m->value() == 0) {
-          if (String_Quoted* ss = dynamic_cast<String_Quoted*>(s)) {
-            if(!ss->quote_mark()) return new (ctx.mem) Null(pstate);
-          } else {
-            return new (ctx.mem) Null(pstate);
-          }
-          newstr = "";
-        } else if(start == end && m->value() != 0) {
-          newstr = str.substr(start, 1);
-        } else if(end > start) {
-          newstr = str.substr(start, end - start + UTF_8::code_point_size_at_offset(str, end));
+        if (start_at <= end_at)
+        {
+          string::iterator start = str.begin();
+          utf8::advance(start, start_at - 1, str.end());
+          string::iterator end = start;
+          utf8::advance(end, end_at - start_at + 1, str.end());
+          newstr = string(start, end);
         }
         if (String_Quoted* ss = dynamic_cast<String_Quoted*>(s)) {
           if(ss->quote_mark()) newstr = quote(newstr);
@@ -1110,7 +1107,7 @@ namespace Sass {
 
       List* list = dynamic_cast<List*>(env["$list"]);
       return new (ctx.mem) Number(pstate,
-                                  list ? list->length() : 1);
+                                  list ? list->size() : 1);
     }
 
     Signature nth_sig = "nth($list, $n)";
@@ -1220,7 +1217,18 @@ namespace Sass {
       else if (sep_str == "comma") result->separator(List::COMMA);
       else if (sep_str != "auto") error("argument `$separator` of `" + string(sig) + "` must be `space`, `comma`, or `auto`", pstate);
       *result += l;
-      *result << v;
+      bool is_arglist = l->is_arglist();
+      result->is_arglist(is_arglist);
+      if (is_arglist) {
+        *result << new (ctx.mem) Argument(v->pstate(),
+                                          v,
+                                          "",
+                                          false,
+                                          false);
+
+      } else {
+        *result << v;
+      }
       return result;
     }
 
@@ -1254,13 +1262,6 @@ namespace Sass {
       return zippers;
     }
 
-    Signature compact_sig = "compact($values...)";
-    BUILT_IN(compact)
-    {
-      error("`compact` has been removed from libsass because it's not part of the Sass spec", pstate);
-      return 0; // suppress warning, error will exit anyway
-    }
-
     Signature list_separator_sig = "list_separator($list)";
     BUILT_IN(list_separator)
     {
@@ -1355,13 +1356,10 @@ namespace Sass {
     {
       List* arglist = new (ctx.mem) List(*ARG("$args", List));
       Map* result = new (ctx.mem) Map(pstate, 1);
-      // The parser ensures the ordering of arguments so we can assert this
-      // isn't keyword argument list the first argument isn't a keyword argument
-      if (!(arglist->empty() || ((Argument*)(*arglist)[0])->is_keyword_argument())) return result;
-      for (size_t i = 0, L = arglist->length(); i < L; ++i) {
+      for (size_t i = arglist->size(), L = arglist->length(); i < L; ++i) {
         string name = string(((Argument*)(*arglist)[i])->name());
-        string sanitized_name = string(name, 1);
-        *result << make_pair(new (ctx.mem) String_Constant(pstate, sanitized_name),
+        name = name.erase(0, 1); // sanitize name (remove dollar sign)
+        *result << make_pair(new (ctx.mem) String_Constant(pstate, name),
                              ((Argument*)(*arglist)[i])->value());
       }
       return result;
@@ -1550,11 +1548,14 @@ namespace Sass {
       } else if (v->concrete_type() == Expression::STRING) {
         return v;
       } else {
+        bool parentheses = v->concrete_type() == Expression::MAP ||
+                           v->concrete_type() == Expression::LIST;
         Output_Style old_style;
         old_style = ctx.output_style;
         ctx.output_style = NESTED;
-        To_String to_string(&ctx);
+        To_String to_string(&ctx, false);
         string inspect = v->perform(&to_string);
+        if (inspect.empty() && parentheses) inspect = "()";
         ctx.output_style = old_style;
         return new (ctx.mem) String_Constant(pstate, inspect);
 
@@ -1563,6 +1564,20 @@ namespace Sass {
       // return v;
     }
 
+    Signature is_superselector_sig = "is-superselector($super, $sub)";
+    BUILT_IN(is_superselector)
+    {
+      To_String to_string(&ctx, false);
+      Expression*  ex_sup = ARG("$super", Expression);
+      Expression*  ex_sub = ARG("$sub", Expression);
+      string sup_src = ex_sup->perform(&to_string) + "{";
+      string sub_src = ex_sub->perform(&to_string) + "{";
+      Selector_List* sel_sup = Parser::parse_selector(sup_src.c_str(), ctx);
+      Selector_List* sel_sub = Parser::parse_selector(sub_src.c_str(), ctx);
+      bool result = sel_sup->is_superselector_of(sel_sub);
+      return new (ctx.mem) Boolean(pstate, result);
+    }
+
     Signature unique_id_sig = "unique-id()";
     BUILT_IN(unique_id)
     {
