@@ -12,7 +12,7 @@ namespace Sass {
 
     using namespace std;
 
-    struct Context;
+    class Context;
 
     class Remove_Placeholders : public Operation_CRTP<void, Remove_Placeholders> {
 
