@@ -262,21 +262,15 @@ namespace Sass {
   {
     bool ws = false;
     bool esc = false;
-    char inside_str = 0;
     string text = "";
-    for(auto i : str) {
+    for(const char& i : str) {
       if (!esc && i == '\\') {
         esc = true;
+        ws = false;
         text += i;
       } else if (esc) {
         esc = false;
-        text += i;
-      } else if (!inside_str && (i == '"' || i == '\'')) {
-        inside_str = i;
-        text += i;
-      } else if (inside_str) {
-        if (i == inside_str)
-          inside_str = false;
+        ws = false;
         text += i;
       } else if (
         i == ' ' ||
@@ -401,7 +395,7 @@ namespace Sass {
 
   }
 
-  string quote(const string& s, char q)
+  string quote(const string& s, char q, bool keep_linefeed_whitespace)
   {
 
     // autodetect with fallback to given quote
@@ -430,6 +424,10 @@ namespace Sass {
       if (cp == 10) {
         quoted.push_back('\\');
         quoted.push_back('a');
+        // we hope we can remove this flag once we figure out
+        // why ruby sass has these different output behaviors
+        if (keep_linefeed_whitespace)
+          quoted.push_back(' ');
       } else if (cp < 127) {
         quoted.push_back((char) cp);
       } else {
